//a_stream.h - output classes

///////////////////////////////////////////////////////////////////////////////////
// AStreamOutput class, includes encryption
///////////////////////////////////////////////////////////////////////////////////
class AStreamOutput : public ABase
{
  public:
    AStreamOutput(ostream *posOut = NULL) { m_posOut = (posOut ? posOut : &cout); }
    ~AStreamOutput() {};

    //a_Declares debug/dump related functions
    #ifdef _DEBUG_DUMP_
      void dump(void);     //a_Debugging dump, when _DEBUG_DUMP_ is set
    #endif

    //a_Output to ostream variable m_posOutput, by default cout
    void setOStream(ostream *posNewOut)
    {
      if (posNewOut) m_posOut = posNewOut;
    }
    ostream *getOStream(void) { return m_posOut; }

    //a_Standard stream support with assert() checks
    AStreamOutput &operator<< (char cOut)   { assert(m_posOut); *m_posOut << cOut;  return *this; }
    AStreamOutput &operator<< (short nOut)  { assert(m_posOut); *m_posOut << nOut;  return *this; }
    AStreamOutput &operator<< (long lOut)   { assert(m_posOut); *m_posOut << lOut;  return *this; }
    AStreamOutput &operator<< (BYTE bOut)   { assert(m_posOut); *m_posOut << bOut;  return *this; }
    AStreamOutput &operator<< (WORD wOut)   { assert(m_posOut); *m_posOut << wOut;  return *this; }
    AStreamOutput &operator<< (DWORD dwOut) { assert(m_posOut); *m_posOut << dwOut; return *this; }
    AStreamOutput &operator<< (int iOut)    { assert(m_posOut); *m_posOut << iOut;  return *this; }
    AStreamOutput &operator<< (UINT uOut)   { assert(m_posOut); *m_posOut << uOut;  return *this; }
    AStreamOutput &operator<< (float fOut)  { assert(m_posOut); *m_posOut << fOut;  return *this; }
    AStreamOutput &operator<< (double dOut) { assert(m_posOut); *m_posOut << dOut;  return *this; }
    AStreamOutput &operator<< (const char *pccOut)
      { assert(m_posOut); outString(pccOut); return *this; }
    AStreamOutput &operator<< (signed char scOut)
      { assert(m_posOut); *m_posOut << scOut;  return *this; }
    AStreamOutput &operator<< (void *pv)
      { assert(m_posOut); *m_posOut << pv; return *this; }
    AStreamOutput &operator<< (streambuf *psb)
      { assert(m_posOut); *m_posOut << psb; return *this; }
    AStreamOutput &operator<< (ostream& (*fcn)(ostream&))
      { assert(m_posOut); *m_posOut << fcn; return *this; }
    AStreamOutput &operator<< (ios& (*fcn)(ios&))
      { assert(m_posOut); *m_posOut << fcn; return *this; }

    //a_Stream wrappers
    void outString(const char *pccOut)
    {
      assert(m_posOut);
      if (pccOut)
        *m_posOut << pccOut;
      else
        *m_posOut << "(null)";
    }
    void outStringN(const char *pccOut)  //a_For speed does no check
    {
      assert(m_posOut);
      *m_posOut << pccOut;
    }
    void outStringQ(const char *pccOut)  //a_Add quotes
    {
      assert(m_posOut);
      if (pccOut)
        *m_posOut << "\"" << pccOut << "\"";
      else
        *m_posOut << "(null)";
    }
    void outStringCR(const char *pccOut)
    {
      assert(m_posOut);
      if (pccOut)
        *m_posOut << pccOut;
      else
        *m_posOut << "(null)";
    
      *m_posOut << endl;            //a_With end-of-line added
    }
    void outStringCRN(const char *pccOut)
    {
      assert(m_posOut);
      *m_posOut << pccOut << endl;
    }

    #if defined(_WINDOWS) && defined(_MSC_VER)  //a_Used only if Visual C++ is used with CWinApp
      #pragma message("CWinApp definition found: streamCreateConsoleStdio() is available.")
      //a_Allocates a console session and attaches iostream to stdio of the console
      int streamCreateConsoleStdio(int iSetOutStreamBinary = 0x0);
    #endif

  protected:
    ostream *m_posOut;

    //a_Raw dump to the output stream (what ever it was set to)
};


///////////////////////////////////////////////////////////////////////////////////
// AHTML class
///////////////////////////////////////////////////////////////////////////////////
class AHTML : public AStreamOutput
{
  public:
    AHTML(ostream *posOut = NULL) : AStreamOutput(posOut) {}
    virtual ~AHTML() {}
    
    //a_Declares debug/dump related functions
    #ifdef _DEBUG_DUMP_
      void dump(void);     //a_Debugging dump, when _DEBUG_DUMP_ is set
    #endif

    //a_NOTE: \n added to make output readable to the user and is not needed for HTML
    //a_HERE IS A BASIC HTML DOCUMENT (Note: Must send the MIME directive first from a CGI)
    //a_<HTML>
    //a_<HEAD><TITLE>My Title</TITLE></HEAD>
    //a_<BODY>
    //a_</BODY>
    //a_</HTML>

    //a_HTML start & end
    void htmlStartHTML(void)      { outStringCRN("<HTML>"); };
    void htmlEndHTML(void)        { outStringCRN("</HTML>"); };
    
    //a_HEAD section and TITLE; htmlDoHEAD() is all that is really needed at this time
    void htmlStartHEAD(void)      { outStringCRN("<HEAD>"); };
    void htmlEndHEAD(void)        { outStringCRN("</HEAD>"); };
    void htmlDoHEAD(const char *pccTitle, const char *pccExtra = NULL);

    void htmlStartTITLE(void)     { outStringCRN("<TITLE>"); };
    void htmlEndTITLE(void)       { outStringCRN("</TITLE>"); };
    void htmlDoTITLE(const char *pccTitle);

    //a_META directive
    void htmlDoMETA(const char *pccName, const char *pccContent);

    //a_BODY directives
    void htmlStartBODY(COLORREF crBack = 0x00808080, COLORREF crText = 0x00FFFFFF,
                       COLORREF crLink = 0x0000DDDD, COLORREF crALink = 0x0000FFFF,
                       COLORREF crVLink = 0x00000000, const char *pccBack = NULL,
                       const char *pccExtra = NULL);
    void htmlStartBODY(AElementPairList &eplBody);
    void htmlEndBODY(void)        { outStringCRN("</BODY>"); }

    //a_Format: <pccTag pccExtra> {pccText} </pccTag>\n and similar
    void htmlStartTag(const char *pccTag, const char *pccExtra = NULL);
    void htmlStartTag(const char *pccTag, AElementPairList &eplTag);
    void htmlEndTag(const char *pccTag);
    void htmlDoTag(const char *pccTag, const char *pccText);
    void htmlDoTagEx(const char *pccTag, AElementPairList &eplTag, const char *pccText);
    void htmlDoTagEx(const char *pccTag, const char *pccExtra, const char *pccText);

    //a_Scripting support
    void htmlStartSCRIPT(const char *pccLanguage = NULL);
    void htmlEndSCRIPT();

    //a_Space skipping using &nbsp; control character
    void htmlDoSpace(int iN = 0x1);
    
    //a_The comment <!-- ... -->
    void htmlDoComment(const char *pcc1, const char *pcc2 = NULL, const char *pcc3 = NULL);

    //a_Helper functions
    void htmlDateTime(void)
    {
      time_t ttNow = time(NULL);
      outString(ctime(&ttNow));
    }

    //a_Generic simple MIME output (type/subtype  and content length for binary types such as GIF or JPEG)
    void mimeOut(const char *pccType, const char *pccSubType, int iContentLength = -1);
    //a_MIME directive for HTML output with HTTP Cookie support
    void mimeHTML(ACookie *paCookie = NULL);
    //a_Only callable by AXBitmap (so far) when outputing x-xbitmap
    void mimeXBitmap(void)
    { 
      mimeOut("image", "x-xbitmap");
      //outStringN("Content-Type: image/x-xbitmap\xA\xA");
    }
};

//a_Global redirection operators (to avoid declaration matching), uses polymorphism of AElement
inline AStreamOutput &operator <<(AStreamOutput &ahtmlOut, const ABaseElement &abeSource)
{
  abeSource.doOut(&ahtmlOut);
  return ahtmlOut;
}


