////////////////////////////
//                        //
// ACGI++ Class Hierarchy //
//                        //
///////////////////////////////////////////////////////////////
// a_bitary.cpp                                              //
//                                                           //
// Classes to be used in CGI development using C++           //
//                                                           //
// Copyright of Quasit Technologies, Inc.                    //
//                                                           //
///////////////////////////////////////////////////////////////
#include "a_acgi.h"


#pragma message( "Compiling " __FILE__ )

//----------------------------------------------------------------------------------------

#ifdef _DEBUG_DUMP_
void ABitArray::dump(void)
{
  cout << endl << "---START: ABitArray::dump()---";
  AArray::dump();                                 //a_Do parent dump as well

  //a_ABitArray contents
  if (m_pData)
  {
    int iEnd = aGetTrueSize() - 1, iX;
    BYTE bX;
    for (iX = 0x0; iX < iEnd; iX++)
    {
      bX = BYTE(*(m_pData + iX));
      
      cout << "0x" << hex << int(bX);

      if (((iX + 1) % 0x8) == 0x0)
        cout << "," << endl;
      else
        cout << ", ";
    }

    bX = BYTE(*(m_pData + iX));
    
    cout << "0x" << hex << int(bX);   //a_NOTE: No last endl!
  }

  cout << endl << "---END: ABitArray::dump()---" << endl;
}
#endif

//----------------------------------------------------------------------------------------

ABitArray::ABitArray(ABitArray *pbaSource)
{
  if (pbaSource)
  {
    //a_Copy constructor
    _bCopy(*pbaSource);
  }
}

ABitArray::~ABitArray(void)
{
}

void ABitArray::aSetAt(int iBitPos, int iState, int iMode)
{
  assert(iBitPos < aGetSize() && iBitPos >= 0x0);

  _baSetABit(iBitPos);              //a_Set the ABit
  switch (iMode)
  {
    case BLIT_COPY :
      m_bIt = (iState ? 0x1 : 0x0);
    break;

    case BLIT_AND :
      m_bIt &= (iState ? 0x1 : 0x0);
    break;

    case BLIT_OR :
      m_bIt |= (iState ? 0x1 : 0x0);
    break;

    case BLIT_XOR :
      m_bIt ^= (iState ? 0x1 : 0x0);
    break;

    default : assert(0x0);        //a_Unknown mode
 }
}

//----------------------------------------------------------------------------------------

ABit &ABitArray::operator[] (int iBitPos)
{
  _baSetABit(iBitPos);

  return m_bIt;
}

int ABitArray::aSetSize(int iNewBitSize, int iRealloc)
{
  assert(iNewBitSize > 0x0 && iNewBitSize < INT_MAX);

  int iNewAlloc = (iNewBitSize + 0x7) / 0x8;              //a_Number of bits to BYTEs (rounded up)
  
  if (iNewAlloc > aGetAllocSize() || iRealloc || !m_pData)
  {
    //a_Need to reallocate and reassign from original
    _aSetToBYTEArray(aGetArray(), iNewAlloc);
  }
  else
  {
    //a_Just shrink, without reallocating
    m_iSize = _aMapAllocToSize(iNewAlloc); 
    return 0x1;
  }

  return 0x0;               //a_Failed
}
