////////////////////////////
//                        //
// ACGI++ Class Hierarchy //
//                        //
///////////////////////////////////////////////////////////////
// a_html.cpp                                                //
//                                                           //
// Classes to be used in CGI development using C++           //
//                                                           //
// Copyright of Quasit Technologies, Inc.                    //
//                                                           //
///////////////////////////////////////////////////////////////
#include "a_acgi.h"

#pragma message( "Compiling " __FILE__ )

#ifdef _DEBUG_DUMP_
void AHTML::dump(void)
{
  cout << endl << "---START: AHTML::dump()---";
  AStreamOutput::dump();                                 //a_Do parent dump as well
  cout << endl << "---END: AHTML::dump()---" << endl;
}
#endif

void AHTML::mimeOut(const char *pccType, const char *pccSubType, int iContentLength)
{
  if (pccType && pccSubType)
  {
    if (pccType && pccSubType)
    {
      *this << "Content-Type: " << pccType << "/" << pccSubType << "\xA";
    
      if (iContentLength >= 0x0)
        *this << "Content-Length: " << iContentLength << "\xA\xA" << flush;
      else
        outStringN("\xA\xA");
    }
  }
  else
    assert(0x0);
}

//a_mime directive with Cookie support
void AHTML::mimeHTML(ACookie *paCookie)
{
  outStringN("Content-Type: text/html\xA");
  
  if (paCookie)
  {
    paCookie->doOut(this);         //a_Dump it
    outStringN("\xA");
  }
  outStringN("\xA");
}


void AHTML::htmlDoHEAD(const char *pccTitle, const char *pccExtra)
{
  outStringN("<HEAD><TITLE>");
  outString(pccTitle);
  if (pccExtra)
    outString(pccExtra);               //a_Convenience append string
  outStringCRN("</TITLE></HEAD>");
}

void AHTML::htmlDoTITLE(const char *pccTitle)
{
  outStringN("<TITLE>");
  outString(pccTitle);
  outStringCRN("</TITLE>");
}

//a_HTML output format as follows:
//a_<BODY BACGROUNG=(pccBack) BGCOLOR="crBack" TEXT="crText" LINK="crLink" ALINK="crALink" VLINK="crVLink">
//a_ie. VistitedLINK(if visitied) and ActiveLINK(while clicked)
void AHTML::htmlStartBODY(COLORREF crBack, COLORREF crText, COLORREF crLink, COLORREF crALink, COLORREF crVLink, const char *pccBack, const char *pccExtra)
{
  outStringN("<BODY");
  if (pccBack)
  {
    outStringN(" BACKGROUND=");
    outStringQ(pccBack);
  }

  //a_Now do the colors (temp local buffer is to insure re-entrant code for the function called)
  char sColor[8];
  outStringN(" BGCOLOR=");
  outStringQ(g_aCrypto.convertoCOLORREFtoChar(crBack, sColor));
  outStringN(" TEXT=");
  outStringQ(g_aCrypto.convertoCOLORREFtoChar(crText, sColor));
  outStringN(" LINK=");
  outStringQ(g_aCrypto.convertoCOLORREFtoChar(crLink, sColor));
  outStringN(" ALINK=");
  outStringQ(g_aCrypto.convertoCOLORREFtoChar(crALink, sColor));
  outStringN(" VLINK=");
  outStringQ(g_aCrypto.convertoCOLORREFtoChar(crVLink, sColor));
  if (pccExtra)
  {
    outStringN(" ");
    outString(pccExtra);
  }
  outStringCRN(">");
}

void AHTML::htmlStartBODY(AElementPairList &eplBody)
{
  outStringN("<BODY ");
  *this << " " << eplBody;
  outStringCRN(">");
}

void AHTML::htmlStartTag(const char *pccTag, const char *pccExtra)
{ 
  if (pccTag)
  {
    outStringN("<");
    outString(pccTag);
    if (pccExtra)
    {
      outStringN(" ");
      outString(pccExtra);
    }
    outStringN(">");
  }
}

void AHTML::htmlStartTag(const char *pccTag, AElementPairList &eplTag)
{
  if (pccTag)
  {
    outStringN("<");
    outString(pccTag);
    *this << " " << eplTag;
    outStringCRN(">");
  }
}

void AHTML::htmlEndTag(const char *pccTag)
{ 
  if (pccTag)
  {
    outStringN("</");
    outString(pccTag);
    outStringN(">");
  }
}

void AHTML::htmlDoTag(const char *pccTag, const char *pccText)
{ 
  htmlDoTagEx(pccTag, NULL, pccText);
}

void AHTML::htmlDoTagEx(const char *pccTag, const char *pccExtra, const char *pccText)
{ 
  if (pccText)
  {
    htmlStartTag(pccTag, pccExtra);
    outString(pccText);
    htmlEndTag(pccTag);
  }
}

void AHTML::htmlDoTagEx(const char *pccTag, AElementPairList &eplTag, const char *pccText)
{ 
  if (pccText)
  {
    htmlStartTag(pccTag, eplTag);
    outString(pccText);
    htmlEndTag(pccTag);
  }
}

void AHTML::htmlDoSpace(int iN)
{
  while (iN-- > 0x0)
  {
    //a_13 spaces per line
    if ((iN % 13) == 0x0) outStringCRN("&nbsp;");
    else                  outStringN("&nbsp;");
  }
}

void AHTML::htmlDoComment(const char *pcc1, const char *pcc2, const char *pcc3)
{
  //a_Embed a bunch of text inside a comment (Shouldn't contain "-->" in the pccN's...)
  outStringN("<!--");
    if (pcc1) outString(pcc1);
    if (pcc2) outString(pcc2);
    if (pcc3) outString(pcc3);
  outStringCRN("-->");
}


void AHTML::htmlStartSCRIPT(const char *pccLanguage)
{
  outStringN("<SCRIPT LANGUAGE=");
  if (pccLanguage)
    outStringQ(pccLanguage);   //a_A script language
  else
    outStringQ("JavaScript");  //a_Default script language

  outStringCRN("><!-- Hide from older browsers");
}

void AHTML::htmlEndSCRIPT()
{
  outStringCRN("//--></SCRIPT>");
}

void AHTML::htmlDoMETA(const char *pccName, const char *pccContent)
{
  if (pccName && pccContent)
  {
    outStringN("<META NAME=");
    outStringQ(pccName);
    outStringN(" CONTENT=");
    outStringQ(pccContent);
    outStringCRN(">");
  }
  else
  {
    #ifdef _DEBUG_FULL
      htmlDoComment("META directive contained a NULL pointer");
    #endif
  }
}
