////////////////////////////
//                        //
// ACGI++ Class Hierarchy //
//                        //
///////////////////////////////////////////////////////////////
// a_stream.cpp                                              //
//                                                           //
// Classes to be used in CGI development using C++           //
//                                                           //
// Copyright of Quasit Technologies, Inc.                    //
//                                                           //
///////////////////////////////////////////////////////////////

#pragma message( "Compiling " __FILE__ )

#include "a_acgi.h"

#ifdef _DEBUG_DUMP_
void AStreamOutput::dump(void)
{
  cout << endl << "---START: AStreamOutput::dump()---";
  ABase::dump();                                 //a_Do parent dump as well
  cout << "m_posOut = " << m_posOut << endl;
  cout << "---END: AStreamOutput::dump()---" << endl;
}
#endif

#if defined(_WINDOWS) && defined(_MSC_VER)  //a_Used only if Visual C++ is used with CWinApp
//a_stdout can be text or binary, so if iSetOutStreamBinary != 0 for binary output
//a_Binary output is useful for outputting non-text based MIME-types (no \n -> \n\r translation)
int AStreamOutput::streamCreateConsoleStdio(int iSetOutStreamBinary)
{
  //a_Handles for stdin and stdout and FILE handle when they are opened
  int iCRT;
  FILE *fileHandle;

  //a_Works when executed from console or file manager type, not program manager
  if (!AllocConsole())
  {
    TRACE0("Unable to connact to a console session.\n\r");
    return 0x0;
  }
  
  //a_Create stdout
  iCRT = _open_osfhandle((long) GetStdHandle(STD_OUTPUT_HANDLE), (iSetOutStreamBinary ? _O_BINARY : _O_TEXT));
  if ((fileHandle = _fdopen(iCRT, "w")) == NULL)
  {  
    TRACE0("Unable to open stdout for write.\n\r");
    return 0x0;
  }
  else
    *stdout = *fileHandle;
  
  //a_Create stdin
  iCRT = _open_osfhandle((long) GetStdHandle(STD_INPUT_HANDLE), _O_TEXT);
  if ((fileHandle = _fdopen(iCRT, "r")) == NULL)
  {
    TRACE0("Unable to open stdin for read.\n\r");
    return 0x0;
  }
  else
    *stdin = *fileHandle;

  //a_Create stderr
  iCRT = _open_osfhandle((long) GetStdHandle(STD_ERROR_HANDLE), _O_TEXT);
  if ((fileHandle = _fdopen(iCRT, "w")) == NULL)
  {
    TRACE0("Unable to open stdin for read.\n\r");
    return 0x0;
  }
  else
    *stderr = *fileHandle;

  //a_Attach stdio to buffers
  if (setvbuf(stdout, NULL, _IONBF, 0) || 
      setvbuf(stdin,  NULL, _IONBF, 0) ||
      setvbuf(stderr, NULL, _IONBF, 0))
  {
    TRACE0("Invalid or illegal buffer specified to stdin/stdout/stderr.\n\r");
    return 0x0;
  }

  //a_Re-sync cout/cin/cerr with stdin/stdout/stderr
  ios::sync_with_stdio();

  return 0x1;
}
#endif